@@ -23,6 +23,8 @@ from copy import deepcopy
 import requests
 import time, datetime
 import pytest
+import random
+import prance
 from cvapianalyser import CommunityEdition
 from openapispecdiff import OpenApiSpecDiff
 
@@ -112,12 +114,19 @@ class CloudvectorDAST(object):
             self.apispec_one_path = None
         self.apispec_two_path = APISpecTwo
         if ".txt" not in self.apispec_two_path:
-            self.openapispec_obj = OpenApiSpecDiff.OpenApiSpecDiff(self.apispec_one_path, self.apispec_two_path)
-
+            try:
+                self.openapispec_obj = OpenApiSpecDiff.OpenApiSpecDiff(self.apispec_one_path, self.apispec_two_path)
+            except prance.ValidationError:
+                with open(self.apispec_two_path) as fobj:
+                    apis_list = json.load(fobj)
+                with open("_tmpswp.txt","w+") as fobj:
+                    fobj.write("\n".join(apis_list.get("paths",{}).keys()))
+                    self.apispec_two_path = "_tmpswp.txt"
         self.ceobj = None
         if ce_host:
             self.ceobj = CommunityEdition.CommunityEdition(ce_host, ce_username, ce_password)
-        if not ce_host and ".txt" in self.apispec_two_path:
+
+        if not ce_host and self.apispec_two_path!="_tmpswp.txt" and ".txt" in self.apispec_two_path:
             print("Need either a valid OpenAPI Spec or Cloudvector controller to collect enough API data to create "
                   "tests!")
             raise SystemExit
@@ -204,11 +213,8 @@ class CloudvectorDAST(object):
                     continue
             if api not in self.respcodes_from_spec:
                 self.respcodes_from_spec[api] = {}
-            if api not in params_info_from_spec:
-                params_info_from_spec[api] = {}
+
             for method, minfo in info.items():
-                if method not in params_info_from_spec[api]:
-                    params_info_from_spec[api][method]={"required":[], "optional":[]}
                 x = {}
                 x["method"] = method.lower()
                 x["host"] = host
@@ -224,6 +230,12 @@ class CloudvectorDAST(object):
                 if host:
                     x["url"] = host + baseurl + api
                 x["url"] = baseurl + api
+                base_url_and_api = x["url"]
+                if x["url"] not in params_info_from_spec:
+                    params_info_from_spec[base_url_and_api] = {}
+                if method not in params_info_from_spec[x["url"]]:
+                    params_info_from_spec[base_url_and_api][method]={"required":[], "optional":[]}
+
                 for _ in x["url"].split("/"):
                     if "{" in _ and "}" in _:
                         _ = str(_).replace("{", "").replace("}", "")
@@ -235,11 +247,14 @@ class CloudvectorDAST(object):
                 for _ in minfo.get("parameters", []):
                     if _.get("required", True):
                         if type(_.get("required")) is bool:
-                            params_info_from_spec[api][method]["required"].append(_.get("name"))
+                            params_info_from_spec[base_url_and_api][method]["required"].append(_.get("name"))
                         else:
-                            params_info_from_spec[api][method]["required"]+=_.get("required",[])
+                            params_info_from_spec[base_url_and_api][method]["required"]+=_.get("required",[])
                     else:
-                        params_info_from_spec[api][method]["optional"].append(_.get("name"))
+                        params_info_from_spec[base_url_and_api][method]["optional"].append(_.get("name"))
+                    params_info_from_spec[base_url_and_api][method]["required"] = list(set(params_info_from_spec[base_url_and_api][method]["required"] ))
+                    params_info_from_spec[base_url_and_api][method]["optional"] = list(set(params_info_from_spec[base_url_and_api][method]["optional"] ))
+
                     if _.get("in") == "header":
                         x["header"][_.get("name")] = ""
                     elif _.get("in") == "query":
@@ -260,6 +275,8 @@ class CloudvectorDAST(object):
                 apis_from_spec.append([x, None])
         if not os.path.exists("tests/data"):
             os.mkdir("tests/data")
+        if os.path.exists("tests/data/params_info_from_spec.json"):
+            os.remove("tests/data/params_info_from_spec.json")
         with open("tests/data/params_info_from_spec.json", "a+") as _file:
             json.dump(params_info_from_spec, _file)
         return apis_from_spec
@@ -597,6 +614,9 @@ class CloudvectorDAST(object):
         else:
             statefull = 0
 
+        if os.environ.get("CVIAST_STATEFUL_FUZZ","3") in ["0", "1"]:
+            statefull = int(os.environ.get("CVIAST_STATEFUL_FUZZ"))
+
         if os.path.exists("tests/data"):
             shutil.rmtree("tests/data")
         shutil.copytree(os.path.join(templates_dir, "data"), "tests/data/")
@@ -808,10 +828,15 @@ class CloudvectorDAST(object):
             except:
                 test_response = None
             try:
-                curl_command_to_try = stdout.split("CURL command to retry:")[-1].split("---------------------------------")[0].replace("\n","")
+                if "CURL command to retry:" in curl_command_to_try:
+                    curl_command_to_try = stdout.split("CURL command to retry:")[-1].split("---------------------------------")[0].replace("\n","")
+                else:
+                    curl_command_to_try = ""
             except:
                 curl_command_to_try = ""
 
+            #print(curl_command_to_try,"\n\n\n", stdout)
+
             if api_name not in consolidated_report:
                 consolidated_report[api_name] = {}
             if api_method not in consolidated_report[api_name]:
@@ -834,6 +859,7 @@ class CloudvectorDAST(object):
             fuzz_metadata = get_metadata_for_fuzzinput(fuzz_sub_type)
 
 
+
             if fuzz_metadata:
                 fuzz_metainfo[fuzz_type] = {
                         "description": fuzz_metadata.get("description"),
@@ -865,7 +891,7 @@ class CloudvectorDAST(object):
                             "remedy": fuzz_remedy,
                             "classification": fuzz_classification,
                             "references": fuzz_references,
-                            "ease_of_explanation": fuzz_ease_of_exploitation,
+                            "ease_of_exploitation": fuzz_ease_of_exploitation,
                             "executions": [{
                                                 "api": api_name,
                                                 "api_method": api_method,
@@ -1005,7 +1031,157 @@ class CloudvectorDAST(object):
                                         fuzz_types=fuzz_types, vulns_stats=vulns_stats, fuzz_metadata=fuzz_metainfo,
                                         detailed_report=os.path.join(os.getcwd(),"CVIAST-detailed_report.html"),
                                         test_lapse_time=str(round(int(report["report"]["summary"]["duration"])/60))+" min(s)").encode("utf-8","ignore"))
-        print("CVIAST report saved in " + os.path.join(os.getcwd(),output_report))
+        print("CVIAST report saved in " + os.path.join(os.getcwd(),output_report)+"\n\n")
+
+
+    @staticmethod
+    def create_test_data_sets(api, method):
+        with open("tests/params_captured.json") as fobj:
+            f = json.load(fobj)
+        params_collection = []
+        for _ in f.get(api, {}).get("200", {}).get(method, {}):
+            params_info = {}
+            for k, v in keypaths(_):
+                if type(v) is list:
+                    for e in v:
+                        if type(e) is dict:
+                            for i, j in keypaths(e):
+                                params_info[i[-1]] = j
+                        else:
+                            params_info[k[-1]] = v
+                else:
+                    params_info[k[-1]] = v
+            params_collection.append(params_info)
+
+        input_params = {}
+        processed_params = []
+        selected_events_data = []
+        for collection in params_collection:
+            for k, v in collection.items():
+                if k.startswith("$"):
+                    continue
+                if k not in input_params:
+                    input_params[k] = {}
+                    continue
+                if type(v) is list:
+                    if len(v) > len(input_params.get(k, {}).get(k, [])):
+                        processed_params.append(k)
+                        if input_params:
+                            new_collection = deepcopy(collection)
+                            new_collection["$comment"] = "Maximum length for List " + str(k) + " is " + str(
+                                len(v)) + " with " \
+                                          "values " + str(v)
+                            input_params[k] = new_collection
+                elif type(v) is bool:
+                    continue
+                elif type(v) is int:
+                    if v > input_params.get(k, {}).get(k, 0):
+                        processed_params.append(k)
+                        if input_params:
+                            new_collection = deepcopy(collection)
+                            new_collection["$comment"] = "Maximum value for " + str(k) + " is " + str(v)
+                            input_params[k] = new_collection
+                elif len(str(v)) > len(str(input_params.get(k, {}).get(k, ""))):
+                    processed_params.append(k)
+                    if input_params:
+                        new_collection = deepcopy(collection)
+                        new_collection["$comment"] = "Maximum value for " + str(k) + " is " + str(
+                            v) + " with length " + str(len(str(v)))
+                        input_params[k] = new_collection
+        selected_events_data += list(input_params.values())
+
+        for collection in params_collection:
+            for k, v in collection.items():
+                if k.startswith("$"):
+                    continue
+                if k not in input_params:
+                    input_params[k] = {}
+                    continue
+                try:
+                    v = int(v)
+                except ValueError:
+                    v = str(v)
+                except TypeError:
+                    pass
+                if type(v) is list:
+                    if len(v) < len(input_params.get(k, {}).get(k, [])):
+                        processed_params.append(k)
+                        if input_params:
+                            new_collection = deepcopy(collection)
+                            new_collection["$comment"] = "Minimum length for List " + str(k) + " is " + str(
+                                len(v)) + " with " \
+                                          "values " + str(v)
+                            input_params[k] = new_collection
+                elif type(v) is bool:
+                    continue
+                elif type(v) is int:
+                    if v < int(input_params.get(k, {}).get(k, 0)):
+                        processed_params.append(k)
+                        if input_params:
+                            new_collection = deepcopy(collection)
+                            new_collection["$comment"] = "Minimum value for " + str(k) + " is " + str(v)
+                            input_params[k] = new_collection
+                elif len(str(v)) < len(str(input_params.get(k, {}).get(k, ""))):
+                    processed_params.append(k)
+                    if input_params:
+                        new_collection = deepcopy(collection)
+                        new_collection["$comment"] = "Minimum value for " + str(k) + " is " + str(
+                            v) + " with length " + str(len(str(v)))
+                        input_params[k] = new_collection
+        selected_events_data += list(input_params.values())
+        return selected_events_data
+
+    @staticmethod
+    def generate_auto_input_params_file():
+        if not os.path.exists("tests") or not os.path.exists("tests/params_captured.json"):
+            print("Error: Run --generate-input-params from the folder that contains the tests folder and tests/params_captured.json file")
+        try:
+            with open("tests/data/params_info_from_spec.json") as fobj:
+                f1 = json.load(fobj)
+        except FileNotFoundError:
+            print("Error: You need to have used a valid SPEC to generate the tests to know the parameter details")
+        input_params = {}
+        for api, info in f1.items():
+            if api not in input_params:
+                input_params[api] = {"200": {}, "400": {}, "401": {}}
+            for method, method_info in info.items():
+                data_set = CloudvectorDAST.create_test_data_sets(api, method)
+                if not data_set:
+                    continue
+                random_data_set = random.choice(data_set)
+                for k, v in random_data_set.items():
+                    if k.startswith("$"):
+                        continue
+                    if k in method_info.get("required"):
+                        error_data_block = deepcopy(random_data_set)
+                        error_data_block[k] = "<delete>"
+                        error_data_block["$comment"] = "This payload will miss the mandatory parameter " + str(k)
+                        if method not in input_params[api]["400"]:
+                            input_params[api]["400"][method] = [error_data_block]
+                        else:
+                            input_params[api]["400"][method].append(error_data_block)
+                    else:
+                        success_data_block = deepcopy(random_data_set)
+                        success_data_block[k] = "<delete>"
+                        success_data_block["$comment"] = "This payload will miss the optional parameter " + str(k)
+                        if method not in input_params[api]["200"]:
+                            input_params[api]["200"][method] = [success_data_block]
+                        else:
+                            input_params[api]["200"][method].append(success_data_block)
+
+                if method not in input_params[api]["200"]:
+                    input_params[api]["200"][method] = data_set
+                else:
+                    input_params[api]["200"][method] += data_set
+
+        if os.path.exists("tests"):
+            with open("tests/input_params_auto-generated.json", "w+") as fobj:
+                fobj.write(json.dumps(input_params, indent=4))
+            print("\n\tAuto generated input params saved in "+os.path.join(os.getcwd(),os.path.join("tests", "input_params_auto-generated.json")))
+        else:
+            with open("input_params_auto-generated.json", "w+") as fobj:
+                fobj.write(json.dumps(input_params, indent=4))
+            print("\n\tAuto generated input params saved in "+os.path.join(os.getcwd(),"input_params_auto-generated.json"))
 
 def printProgressBar (iteration, total, prefix = '', suffix = '', decimals = 1, length = 100, fill = '█', printEnd = "\r"):
     """
@@ -1053,11 +1229,17 @@ def main():
         if sys.argv[1:][0] == "--generate-report":
             CloudvectorDAST.generate_fuzz_report(html_report_name)
             return
+        if sys.argv[1:][0] == "--generate-input-params":
+            CloudvectorDAST.generate_auto_input_params_file()
+            return
         if sys.argv[1:][0] == "--execute":
             arguments = sys.argv[2:]
             arguments += ["--json=report.json", "--html=CVIAST-detailed_report.html", "--self-contained-html", "-vv"]
+            #arguments += ["--json=report.json", "--html=CVIAST-detailed_report.html", "--self-contained-html", "-vv", ">> cviast_execution_log.txt"]
             try:
+                print("\n....Test Execution is in progress.............\n\n")
                 pytest.main(arguments)
+                #os.system("pytest "+" ".join(arguments))
                 CloudvectorDAST.generate_fuzz_report(html_report_name)
             except:
                 pass
@@ -1125,6 +1307,7 @@ def main():
         if sys.argv[1:][0] == "--fuzz":
             print("\nFuzzing enabled!\n")
             enable_fuzzing = True
+
     CloudvectorDAST(str(input_spec_one).strip(), str(input_spec_two).strip(), ce_host, ce_username, ce_password, config,
                     str(cover_only_diff).lower(), input_params_file, enable_fuzzing)
     print(
